
// .-----------------------------------------------------------------------.
// |                                                                       |
// |                          V L A S T N I   E D I T O R                  |
// |                                                                       |
// `-----------------------------------------------------------------------'

#include"editor.h"
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<alloc.h>

TStrings *ClipBoard;
//--------------------------- TStrings::TStrings  -----------------------------------------
// Konstruktor bez parametru
TStrings::TStrings () {
  Head=NULL;
  Tail=NULL;
  Count=0;
} /* TStrings::TStrings */

//--------------------------- TStrings::~Tstrings  -------------------------------------
TStrings::~TStrings () {
  clear();
} /* TStrings::~Tstrings */

//--------------------------- TStrings::new_item  -----------------------------------------
// vytvori novou polozku seznamu - alokuje pro ni misto, nealokuje misto pro
// string Value!
TStringItem *TStrings::new_item (string _Value) {
  TStringItem *Pom;

  // alokuje se misto pro novou polozku
  if ((Pom=(TStringItem*)malloc(sizeof (TStringItem)))==NULL)
    return (NULL);

  Pom->Value=_Value;
  Pom->Next=NULL;
  Pom->Prev=NULL;
  return (Pom);
} /* TStrings::new_item */

//--------------------------- TStrings::add  -----------------------------------------
// prida novou polozku s hodnotou Value na konec seznamu
void TStrings::add (string Value) {
  TStringItem *Pom=new_item (Value);
  if (Pom!=NULL){
    if (Head==NULL && Tail==NULL) {
      Head=Pom;
      Tail=Pom;
    }
    else {
      Tail->Next=Pom;
      Pom->Prev=Tail;
      Tail=Pom;
    }
    Count++;
  }
  else
    set_error (1);
} /* TStrings::add */

//--------------------------- TStrings::p_item  -----------------------------------------
// vraci ukazatel na polozku s indexem ItemIndex
TStringItem *TStrings::p_item (long ItemIndex) {
  TStringItem *Pom=NULL;
  if (ItemIndex<Count&&ItemIndex>=0){
    if (ItemIndex<Count/2){
      // je v prvni polovine
      Pom=Head;
      for(long i=0; i<ItemIndex; i++)
	Pom=Pom->Next;
    }
    else {
      // je v druhe polovine
      Pom=Tail;
      for(long i=Count-1; i>ItemIndex; i--)
	Pom=Pom->Prev;
    }// end of else
  }// end of if
  return (Pom);
} /* TStrings::p_item */

//--------------------------- TStrings::item  -----------------------------------------
// vraci hodnootu ItemIndexte polozky seznamu
string TStrings::item (long ItemIndex) {
  return (p_item (ItemIndex)->Value);
} /* TStrings::item */

//--------------------------- TStrings::print  -----------------------------------------
void TStrings::print () {
  TStringItem *Pom=Head;

  for(int i=0; i<Count; i++){
    printf ("%s",Pom->Value);
    Pom=Pom->Next;
  }
} /* TStrings::print */

//--------------------------- TStrings::load  -----------------------------------------
// Pokusi se nacist soubor do seznamu
// vraci:
//   0 - vse OK
//  -1 - soubor nelze otevrit
//  -2 - moc dlouhy soubor
int TStrings::load (string FileName) {
  FILE *F;
  string Line=NULL;
  int Result=0;


  if ((F=fopen (FileName, "rt"))==NULL){
    return (-1);
  }
  else{
    clear();

    while (((Result=read_line (F,Line))==0)&&(Line!=NULL))
      add (Line);
    fclose (F);
    if (Result==-1 || get_error()!=0){
      set_error(0);
      clear();
      return (-2);
    }
    return (0);
  }
} /* TStrings::load */

//--------------------------- TStrings::save  -----------------------------------------
// pokusi se ulozit seznam do souboru
// vraci:
//   0 - vse OK
//  -1 - nepodarilo se ulozit

int TStrings::save (string FileName) {
  FILE *F;
  string Line;
  TStringItem *Pom=Head;



  F=fopen (FileName, "wt");
  if (F!=NULL){
    for(int i=0; i<Count; i++){
      fputs (Pom->Value,F);
      Pom=Pom->Next;
    }
    fclose (F);
    return (0);
  }
  else{
    //Nepodarilo se uloziti soubor
    return (-1);
  }
} /* TStrings::save */

//--------------------------- TStrings::clear  -----------------------------------------
void TStrings::clear () {
  TStringItem *Pom;
  for(int i=0; i<Count; i++){
    Pom=Head;
    Head=Pom->Next;
    free (Pom->Value);
    free (Pom);
  }

  Count=0;
  Head=NULL;
  Tail=NULL;
} /* TStrings::clear */

//--------------------------- TStrings::p_insert  -----------------------------------------
TStringItem *TStrings::p_insert (string Value, TStringItem *NextItem) {
  TStringItem *New=new_item (Value),
	      *Pom;

  // vkladani na prvni posici
  if (NextItem==Head){
    New->Next=NextItem;
    New->Prev=NULL;
    NextItem->Prev=New;
    Head=New;
  }
  // vkladani na konec
  else if (NextItem==NULL){
    New->Next=NULL;
    New->Prev=Tail;
    Tail->Next=New;
    Tail=New;
  }
  // klasicke fkladani doprostred
  else {
    New->Next=NextItem;
    New->Prev=NextItem->Prev;
    NextItem->Prev->Next=New;
    NextItem->Prev=New;
  }

  Count++;
  return (New);
} /* TStrings::p_insert */
//--------------------------- TStrings::insert  -----------------------------------------
TStringItem *TStrings::insert (string Value, long Index) {
  if (Index<0) Index=0;
  if (Index>Count)
    return (p_insert (Value, NULL));
  else
    return (p_insert (Value, p_item (Index)));
} /* TStrings::insert */

//--------------------------- TStrings::p_exchange  -----------------------------------------
void TStrings::p_exchange (TStringItem *Item1, TStringItem *Item2) {
  if (Item1!=NULL && Item2!=NULL){
    string Pom=Item1->Value;
    Item1->Value=Item2->Value;
    Item2->Value=Pom;
  }
} /* TStrings::p_exchange */

//--------------------------- TStrings::exchange  -----------------------------------------
void TStrings::exchange (long ItemIndex1, long ItemIndex2) {
  TStringItem *Item1=p_item (ItemIndex1),
	      *Item2=p_item (ItemIndex2);
  p_exchange (Item1, Item2);
} /* TStrings::exchange */

//--------------------------- TStrings::p_remove  -----------------------------------------
void TStrings::p_remove (TStringItem *Item) {
  TStringItem *Pom=p_cut (Item);
  if (Pom!=NULL) {
    free (Pom->Value);
    free (Pom);
  }
} /* TStrings::p_remove */

//--------------------------- TStrings::remove  -----------------------------------------
void TStrings::remove (long ItemIndex) {
  TStringItem *Pom=cut (ItemIndex);
  if (Pom!=NULL){
    free (Pom->Value);
    free (Pom);
  }
} /* TStrings::remove */

//--------------------------- TStrings::p_cut  -----------------------------------------
TStringItem *TStrings::p_cut (TStringItem *Item) {
  if (Item!=NULL){
    if (Item==Head&&Item==Tail){
      Head=NULL;
      Tail=NULL;
    }
    else if (Item==Head){
      Head=Item->Next;
      Head->Prev=NULL;
    }
    else if (Item==Tail){
      Tail=Item->Prev;
      Tail->Next=NULL;
    }
    else{
      Item->Prev->Next=Item->Next;
      Item->Next->Prev=Item->Prev;
    }
    Count--;
    Item->Next=NULL;
    Item->Prev=NULL;
    return (Item);
  }
  else
    return (NULL);
} /* TStrings::p_cut */

//--------------------------- TStrings::cut  -----------------------------------------
TStringItem *TStrings::cut (long ItemIndex) {
  TStringItem *Pom=p_item(ItemIndex);
  return (p_cut (Pom));
} /* TStrings::cut */

//--------------------------- TStrings::p_split  -----------------------------------------
// rozpoji jednu polozku seznamu na dve
TStringItem *TStrings::p_split (TStringItem *Item, uint Pos) {
  TStringItem *First=Item;
  if (Item!=NULL && Item->Value!=NULL){

    string Value=Item->Value;
    uint N=strlen (Value);

    if (Pos<N){
      string Value1=new_str (Pos+1);
      string Value2=new_str (N-Pos);

      strncpy (Value1,Value,Pos);
      Value1[Pos]='\n';
      strncpy (Value2,&Value[Pos],N-Pos);

      First=p_insert (Value1,Item);
      p_insert (Value2,Item);

      p_remove (Item);
    }
  }
  return (First);
} /* TStrings::p_split */

//--------------------------- TStrings::split  -----------------------------------------
TStringItem *TStrings::split (long ItemIndex, uint Pos) {
  return (p_split (p_item (ItemIndex),Pos));
} /* TStrings::split */

//--------------------------- TStrings::p_join  -----------------------------------------
void TStrings::p_join (TStringItem *FirstItem) {
// spoji dve polozky seznamu, a nezapomina na znak '\n' na konci prvni
// polozky, ktery smaze
  if (FirstItem->Next!=NULL && FirstItem->Value!=NULL &&
      FirstItem->Next->Value!=NULL){
    uint N1=strlen (FirstItem->Value);
    uint N=N1+strlen (FirstItem->Next->Value);

    string Value=new_str (N-1);
    strcpy (Value,FirstItem->Value);
    strcpy (&Value[N1-1],FirstItem->Next->Value);

    p_insert (Value, FirstItem);
    p_remove (FirstItem->Next);
    p_remove (FirstItem);
  }
} /* TStrings::p_join */

//--------------------------- TStrings::join  -----------------------------------------
void TStrings::join (long FirstItemIndex) {
  p_join (p_item (FirstItemIndex));
} /* TStrings::join */


//--------------------------- TStrings::b_set  -----------------------------------------
TBlock TStrings::b_set (long &First, long Last, uint &BBeg, uint BEnd) {
// Vraci vytvorenou strukturu TBlock vzniklou z udaju uvedenych parametry
// Pricemz nejdrive zkontroluje, zda jsou platne a krapet je upravi
// (Zkrati, pokud presahuji rozsah seznamu nebo retezce, Pripadne prehodi
// zacatek a konce bloku)
// Vraci zpet taktez poopravene konstanty First a BBeg

  TBlock Pom;
  TStringItem *PFirst, *PLast;

  // a tady se hnedle zkontroluje, zda jest ten blok platny, nebo zda se ma
  // prevratit a podobne
  if (First>=Count) First=Count-1;
  if (Last>=Count) Last=Count-1;
  if (First<0) First=0;
  if (Last<0) Last=0;


  // pripadne prehodi zacatek a konec bloku
  if ((First>Last)||((First==Last)&&BBeg>BEnd)){
    long PomLine=First;
    uint PomPos=BBeg;

    First=Last;
    Last=PomLine;
    BBeg=BEnd;
    BEnd=PomPos;
  }

  PFirst=p_item (First);  // ukazatel na prvni polozku bloku
  PLast=p_item (Last);    //             posledni


  if (BBeg >= strlen (PFirst->Value))
    BBeg=strlen (PFirst->Value)-1;

  if (BEnd >= strlen (PLast->Value))
    BEnd=strlen (PLast->Value)-1;

  Pom.First=PFirst;
  Pom.Last=PLast;
  Pom.Count=Last-First+1;
  Pom.BBeg=BBeg;
  Pom.BEnd=BEnd;
  return (Pom);
} /* TStrings::b_set */

//--------------------------- TStrings::b_reset  -----------------------------------------
void TStrings::b_reset (TBlock &Block) {
  Block.First=NULL;
  Block.Last=NULL;
  Block.Count=0;
  Block.BBeg=0;
  Block.BEnd=0;
} /* TStrings::b_reset */
//--------------------------------------------------------------------------


//--------------------------- TStrings::b_prepare  -----------------------------------------
void TStrings::b_prepare (TBlock Block, string &FBLine, char &LBChar) {
/* Pripravi pomyslny blok na nejakou blokovou operaci
   Blok je zadan ukazateli na prvni a posledni polozku bloku
   a cisly znacici na ktere poloze v dane polozce zacina (konci) blok
   (kterym pismenem)
   Priprava spociva v tom, ze prvni retezec bloku se zkrati (o prvnich BBeg
   pismen) - nahradi se retezcem &Str[BBeg] a posledni radek bloku se zkrati
   na delku BEnd - znak na posici BEnd+1 v poslednim radku bloku se
   nahradi znakem '\0'(konec radky)

   Predpoklad: pokud blok zacina a konci na stejnem radku, pak se predpoklada
   ze BBeg<BEnd;
*/
  if (Block.First!=NULL && Block.Last!=NULL &&
      Block.First->Value!=NULL && Block.Last->Value!=NULL){
    uint N=0;

    string LLine=Block.Last->Value;   //posledni radek

    N=strlen (LLine);
    if (Block.BEnd>N) Block.BEnd=N;

    LBChar=LLine[Block.BEnd+1];
    LLine[Block.BEnd+1]='\0';

    FBLine=Block.First->Value;   //prvni radek
    N=strlen (FBLine);
    if (Block.BBeg>N) Block.BBeg=N;

    Block.First->Value=&FBLine[Block.BBeg];
  }

} /* TStrings::b_prepare */
//--------------------------- TStrings::b_restore  -----------------------------------------
void TStrings::b_restore (TBlock Block, string FBLine, char LBChar) {
/*  Obnoveni bloku - nahradi se prvni radek bloku celym radkem, a
    taktez posledni radek se nahradi celym radkem
    Predpoklada se, ze hodnoty promennych FBLine a LBChar se zikaly
    volanim metody b_prepare pred provedenim nejake blokove operace
*/
  Block.First->Value=FBLine;
  Block.Last->Value[Block.BEnd+1]=LBChar;
} /* TStrings::b_restore */

//--------------------------- TStrings::b_print  -----------------------------------------
void TStrings::b_print (TBlock Block) {
  string L;
  char Z;
  b_prepare (Block,L,Z);
  TStringItem *Pom=Block.First;
  for(uint i=0; i<Block.Count; i++){
    printf ("%s",Pom->Value);
    Pom=Pom->Next;
  }
  b_restore (Block,L,Z);
} /* TStrings::b_print */
//--------------------------- TStrings::b_case  -----------------------------------------
void TStrings::b_case (TBlock Block, boolean UpCase) {
  string L;
  char Z;
  b_prepare (Block,L,Z);
  TStringItem *Pom=Block.First;
  for(uint i=0; i<Block.Count; i++){
    if (UpCase)
      Pom->Value=strupr (Pom->Value);
    else
      Pom->Value=strlwr (Pom->Value);

    Pom=Pom->Next;
  }
  b_restore (Block,L,Z);
} /* TStrings::b_case */

//--------------------------- TStrings::b_indent  -----------------------------------------
void TStrings::b_indent (TBlock &Block) {
// prida na zacatek kazde radky bloku mezeru
  TStringItem *Pom=Block.First;
  for(uint i=0; i<Block.Count; i++){
    Pom->Value=new_str_insert_char (0,' ',Pom->Value);
    Pom=Pom->Next;
  }
  Block.BBeg++;
  Block.BEnd++;
} /* TStrings::b_indent */

//--------------------------- TStrings::b_unindent  -----------------------------------------
void TStrings::b_unindent (TBlock &Block) {
// pokud to jde ubere ze zacatku kazde radky bloku jednu mezeru
  TStringItem *Pom=Block.First;
  for(uint i=0; i<Block.Count; i++){
    if (Pom->Value[0]==' '){
      Pom->Value=del_char (0,Pom->Value);
      if (Pom==Block.First) Block.BBeg--;
      if (Pom==Block.Last) Block.BEnd--;
    }
    Pom=Pom->Next;
  }
} /* TStrings::b_unindent */

//--------------------------- TStrings::b_delete  -----------------------------------------
void TStrings::b_delete (TBlock &Block) {
// smaze blok :-)
  string Str=Block.First->Value;
  uint Length=strlen (Str);
  string Str2=Block.Last->Value;
  uint Length2=strlen (Str2);

  if (Block.First==Block.Last){
    // Blok obsahuje pouze (cast) JEDEN radek
    string Str=Block.First->Value;
    uint Length=strlen (Str);
    if (Block.BBeg==0 && Block.BEnd==Length-1)
      p_remove (Block.First);
    else
      Block.First->Value=replace_str ("",Str,Block.BBeg,Block.BEnd-Block.BBeg+1);
  }
  else {
    // Odstrani konec prvniho radku (replace_str)
    // Odstrani zacatek posledniho radku (replace_str)
    // Odstrani vse mezi tim (remove)
    if (Block.BBeg!=0){
      Block.First->Value=replace_str ("\n",Str,Block.BBeg,Length-Block.BBeg);
      Block.First=Block.First->Next;
      Block.Count--;
    }
    if (Block.BEnd!=Length2-1){
      Block.Last->Value=replace_str ("",Str2,0,Block.BEnd+1);
      Block.Last=Block.Last->Prev;
      Block.Count--;
    }

    TStringItem *PomFirst;
    for(uint i=0; i<Block.Count; i++){
      PomFirst=Block.First;
      Block.First=Block.First->Next;
      p_remove (PomFirst);
    }

  }
  b_reset (Block);
} /* TStrings::b_delete */
    void b_copy (TBlock Block, TStrings *OutList);
//--------------------------- TStrings::b_copy  -----------------------------------------
void TStrings::b_copy (TBlock Block, TStrings *OutList) {
  // Zkopiruje blok do vystupniho seznamu, ktery nejdrive smaze
  // Predpoklada tedy ze je naalokovan...
  string L;
  char Z;
  b_prepare (Block,L,Z);
  OutList->clear();
  TStringItem *Pom=Block.First;
  for(uint i=0; i<Block.Count; i++){
    OutList->add (new_str_cpy (Pom->Value));
    Pom=Pom->Next;
  }
  b_restore (Block,L,Z);

} /* TStrings::b_copy */
//--------------------------- TStrings::b_paste  -----------------------------------------
void TStrings::b_paste (TStrings *From, TStringItem *Where, uint Pos) {
  if (Where!=NULL && From!=NULL && !From->empty() && Where->Value!=NULL){
    // Vlozi se prvni radek (pokud je Pos>strlen(Where), doplni se ' ')
    uint N=strlen (Where->Value);
    boolean Split=FALSE;

    TStringItem *First=Where;
    if (Pos!=0){
      if (Pos>N-1){
	string Spaces=n_char (Pos-N+1,' ');
	Where->Value=replace_str (Spaces,Where->Value,N-1,0);
	free (Spaces);
      }
      First=p_split (Where,Pos);
      Where=First->Next;
      Split=TRUE;
    }

    TStringItem *FLine=From->Head;
    for(uint i=0; i<From->Count; i++){
      p_insert (new_str_cpy (FLine->Value),Where);
      FLine=FLine->Next;
    }

    if (Split)
      p_join (First);

    string Str=Where->Prev->Value;
    if (Str[strlen(Str)-1]!='\n'){
      Where->Prev->Value=replace_str ("\n",Str,strlen(Str),0);
      p_join (Where->Prev);
    }

  }

} /* TStrings::b_paste */
//--------------------------- TStrings::insert_ch  -----------------------------------------
void TStrings::insert_ch (TStringItem *Where, uint Pos, char Z) {
// vlozi znak Z na posici Pos v polozce seznamu Where, pokud Pos>strlen
// vyplni tento rozdil mezerami
  if (Where!=NULL && Where->Value!=NULL && Pos<MAX_CHARS_PER_LINE){
    uint N=strlen (Where->Value);
    if (Pos>=N){
      string Spaces=n_char (Pos-N+1,' ');
      Where->Value=replace_str (Spaces,Where->Value,N-1,0);
      free (Spaces);
    }
    Where->Value=new_str_insert_char (Pos,Z,Where->Value);
  }
  else if (Where==NULL && empty()){
  // prazdnej seznam
    add (new_str_cpy("\n"));
    insert_ch (Head,Pos,Z);
  }
} /* TStrings::insert_ch */
//--------------------------- TStrings::del_ch  -----------------------------------------
void TStrings::del_ch (TStringItem *Where, uint Pos) {
  if (Where!=NULL && Where->Value!=NULL && Pos<strlen (Where->Value)){
    if (Pos==strlen (Where->Value)-1){
      if (Count==1 && strlen(Where->Value)==1)
	p_remove (Where);
      else
	p_join (Where);
    }
    else{
      Where->Value=del_char (Pos,Where->Value);
    }
  }

} /* TStrings::del_ch */
//----------------------------------------------------------------------------

// .-----------------------------------------------------------------------.
// |                                                                       |
// |                          V L A S T N I   E D I T O R                  |
// |                                                                       |
// `-----------------------------------------------------------------------'

//----------------------------------------------------------------------------

//--------------------------- TEdit::TEdit  -----------------------------------------
TEdit::TEdit (TPoint _A, TPoint _B, string _FileName){
  // Inicializace vlastnich radek dokumentu a clipboardu
  Lines=new TStrings();
//  ClipBoard=new TStrings();


  // Nastaveni rozmeru
  A=_A;
  B=_B;

  HScrollBar=new THScrollBar(point (A.X+14,B.Y-1), point (B.X-3,B.Y));
  VScrollBar=new TVScrollBar(point (B.X-1,A.Y+1), point (B.X,B.Y-1));
  ST=new TSyntaxTable ();
  load_syntax("S_TABLE.INI");

  // nastaveni barvicek
  BackC=BLUE;
  FrameC=WHITE;
  LetterC=YELLOW;
  BlockBackC=LIGHTGRAY;
  BlockLetterC=YELLOW;
  EditResult=-1;
  // nastaveni dalsich promennych
  set_defaults ();
  set_modify (FALSE);
  OldKey=0;


  //if (_FileName==NULL)
  clear ();
  draw ();
  if (_FileName!=NULL)
    load (_FileName);
  // pripadne nahrati souboru (pokud _FileName!=NULL)


} /* TEdit::TEdit */

//--------------------------- TEdit::~TEdit  -----------------------------------------
TEdit::~TEdit () {
  delete Lines;
  delete VScrollBar;
  delete HScrollBar;
  delete ST;
} /* TEdit::~TEdit */


//--------------------------- TEdit::set_defaults  -----------------------------------------
void TEdit::set_defaults () {
  CurPos.X=0;
  CurPos.Y=0;
  FirstCh.X=0;
  FirstCh.Y=0;
} /* TEdit::set_defaults */

//--------------------------- TEdit::set_modify  -----------------------------------------
void TEdit::set_modify (boolean _Modify) {
  if (Modify!=_Modify){
    Modify=_Modify;
    write_modify ();
  }
} /* TEdit::set_modify */

//--------------------------- TEdit::run  -----------------------------------------
void TEdit::run () {
  draw();
  int (Key);
  while (EditResult==-1){
    Key=read_key();
    handle_event (Key);
  }
} /* TEdit::run */

//--------------------------- TEdit::load  -----------------------------------------
void TEdit::load (string _FileName) {
  int Res;
  //string F;

  string NewFileName=_FileName;
  // Zde se urci jmeno nahravaneho souboru
  if (save_changes()!=0){
    if (NewFileName==NULL){
      if ((open_choosefile_dlg(NewFileName))==0)
	NewFileName=NULL;
      // Vyber souboru
    }
    if (NewFileName!=NULL){
      // A ted se pokusime soubor nacist
      Res=Lines->load (NewFileName);
      switch (Res) {
	case 0:  // Vse O.K.
	  prepare_lines ();
	  strcpy (FileName,NewFileName);
	  strupr (FileName);
	  write_file_name ();
	  set_modify (FALSE);
	  set_defaults ();
	  write_cur_pos ();
	  set_insert_mode (TRUE);
	  deselect_block ();
	  // nastav kursor na pocatek
	  // vse vynulovati
	  // vykresli radky
	  write_lines();
	break;
	case -1: // Nepodarilo se nacist
	  message_dlg ("            Soubor nelze otevrit!",1,1);
	  // Vyhod hlasku
	  // pokracujeme, kde jsme zkoncili
	break;
	case -2: // Soubor byl moc dlouhy
	  message_dlg ("   Soubor je pro tento editor prilis velky,            pouzijte jiny editor",1,1);
	  clear ();
	  // vyhod hlasku
	  // vse vynuluj
	  ;
	break;
      }
    }
  } // R!=0
} /* TEdit::load */

//--------------------------- TEdit::save  -----------------------------------------
int TEdit::save () {
  int Res=0;
  if (BlockSelected){
    deselect_block();
    write_lines ();
  }

  if (!strcmp (FileName,"NONAME.TXT"))
    Res=save_as();
  else{
    if (Lines->save(FileName)==0){
      Modify=FALSE;
      write_modify ();
      Res=1;
    }
    else {
      message_dlg ("      Nepodarilo se ulozit soubor",1,1);
      Res=0;
    }
  }
  return (Res);
} /* TEdit::save */

//--------------------------- TEdit::save_as  -----------------------------------------
int TEdit::save_as () {
  int Res=0;
  if (BlockSelected){
    deselect_block();
    write_lines ();
  }
  string Pom;

  if (open_choosefile_dlg (Pom)!=0){
    strcpy (FileName,Pom);
    strupr(FileName);
    write_file_name();

    if (Lines->save(FileName)==0){
      Modify=FALSE;
      write_modify();
      Res=1;
    }
    else {
      message_dlg ("      Nepodarilo se ulozit soubor",1,1);
      Res=0;
    }
    write_cur_pos();

  }
  return (Res);
} /* TEdit::save_as */
//--------------------------- TEdit::save_changes  -----------------------------------------
int TEdit::save_changes () {
  uint R=-1;
  if (Modify){
    R=message_dlg ("         Prejete si ulozit zmeny?",3,2,3,0);
     if (R==2)
       R=save();
  }
  return (R);
} /* TEdit::save_changes */

//--------------------------- TEdit::clear  -----------------------------------------
void TEdit::clear () {
  if (save_changes()!=0){
    Lines->clear();
    check_lines();
    strcpy (FileName,"NONAME.TXT");
    set_defaults();
    set_insert_mode (TRUE);
    set_modify (FALSE);
    // zrusi blok, selectingblock,....
    deselect_block ();
    // prekresli dulezite komponenty
    write_cur_pos();
    write_file_name();
    write_lines ();
  }
} /* TEdit::clear */
//--------------------------- TEdit::prepare_lines  -----------------------------------------
void TEdit::prepare_lines () {
  // zkontroluje, zda je kazdy radek ukoncen znakem \n
  // a taktez nahradi tabelatory prislusnym poctem mezer (8 mezer)

  TStringItem *Pom=Lines->Head;
  for(long i=0; i<Lines->Count; i++){
    // 1) \n na konci radku
    if (Pom->Value[strlen (Pom->Value)-1]!='\n')
      Pom->Value=replace_str ("\n",Pom->Value,strlen (Pom->Value),0);
    // 2) Tabelatory
    string Pos=NULL;
    while ((Pos=strchr(Pom->Value,'\t'))!=NULL)
      Pom->Value=replace_str("        ",Pom->Value,Pos-Pom->Value,1);
    Pom=Pom->Next;
  }
} /* TEdit::prepare_lines */
//--------------------------- TEdit::close  -----------------------------------------
int TEdit::close () {
  // if (Modify) zeptejse, zda ulozit
  //....
  // Vlastni ukonceni editoru
  if (save_changes()!=0){
    EditResult=0;
//    textattr (attrib(BLACK,LIGHTGRAY));
//    clrscr();
  }
  return (EditResult);
} /* TEdit::close */

//=========================== Procedurky na vykreslovani =====================
//--------------------------- TEdit::draw  -----------------------------------------
void TEdit::draw () {
  // KOMPLETNI vykresleni plochy
  // 1) Vykresleni pozadi
  // 2) Vykresleni nekterych udaju (FileName,Modify,CurPos)
  // 3) Vykresleni radek editovaneho dokumentu

  // ad 1)
 // draw_rect (A,B,attrib (BackC,LetterC));
  draw_frame (A,B,FR_DOUBLE,NULL,BackC,FrameC);
  HScrollBar->draw();
  VScrollBar->draw();
  // ad 2)
  write_file_name ();
  write_cur_pos ();
  write_modify ();
  // ad 3)
  write_lines ();
} /* TEdit::draw */

//--------------------------- TEdit::write_file_name  -----------------------------------------
void TEdit::write_file_name () {
  int Attr=attrib (BackC,FrameC);
  print_xyla (A.X+1,A.Y,B.X-A.X-2,"",Attr);
  print_xyla (A.X+2,A.Y,1," ",Attr);
  print_xyla (A.X+3,A.Y,min (strlen (FileName),B.X-A.X-6),FileName,Attr);
  print_xyla (A.X+3+min(strlen (FileName),B.X-A.X-6),A.Y,1," ",Attr);
} /* TEdit::write_file_name */

//--------------------------- TEdit::write_cur_pos  -----------------------------------------
void TEdit::write_cur_pos () {
  TAttrCur AC=save_ac();
  TPoint Pos=akt_pos();
  goto_xy (A.X+5,B.Y-1);
  textattr (attrib (BackC,FrameC));
  cprintf ("");
  goto_xy (A.X+5,B.Y-1);
  cprintf (" %d:%d ",Pos.X,Pos.Y);

  VScrollBar->redraw (Lines->Count,Pos.Y);
  HScrollBar->redraw (MAX_CHARS_PER_LINE,Pos.X);

  restore_ac (AC);
  goto_xy (A.X+1+CurPos.X, A.Y+1+CurPos.Y);
} /* TEdit::write_cur_pos */

//--------------------------- TEdit::write_modify  -----------------------------------------
void TEdit::write_modify () {
  TAttrCur AC=save_ac();
  goto_xy (A.X+2,B.Y-1);
  textattr (attrib (BackC,FrameC));
  if (Modify)
    cprintf ("");
  else
    cprintf ("");
  restore_ac (AC);
} /* TEdit::write_modify */

//--------------------------- TEdit::p_write_line  -----------------------------------------
void TEdit::p_write_line (TStringItem *Item, uint _Y, boolean Bl) {
  if (Item!=NULL){
    string Str=Item->Value;
    uint X=A.X+1;
    uint Y=A.Y+1+_Y;
    uint Length=B.X-A.X-2;
    uint i=0;
    uint N=0;

    if (Bl) block_attr();
    char DelLine[]="                                                                              ";
    //"                                                                              ";
    //""

    if (X<MAX_X && Y<MAX_Y){

      goto_xy (X,Y);
      if (Bl){
	if (strlen (Str)-1>FirstCh.X) Str=&Str[FirstCh.X];
	else Str=NULL;

	if (Str!=NULL){
	  N=strlen (Str)-1;  // bez odradkovani na konci
	  uint Pos=min (Length,N);
	  char Z=Str[Pos];
	  Str[Pos]='\0';
	  cprintf("%s",Str);
	  Str[Pos]=Z;
	  i=Pos;
	}
      }
      else{
	if (Str!=NULL && strlen (Str)-1>FirstCh.X){
	  N=strlen (Str)-1;  // bez odradkovani na konci
	  Str[N]='\0';
	  i=min (Length,N-FirstCh.X)-1;
	  //i=11;
	  ST->my_cprint (Str,FirstCh.X,FirstCh.X+i);
	  Str[N]='\n';
	  normal_attr ();
	  i++;
	  //goto_xy (X+i,Y);
	}
      }
      if (Bl) normal_attr();
      if (i<Length){
	DelLine[Length-i]='\0';
	cprintf(DelLine);
      }
    }
  }
} /* TEdit::p_write_line */

//--------------------------- TEdit::write_block_line  -----------------------------------------
void TEdit::write_block_line (TStringItem *Item, uint Y) {
  boolean First=(Item==Block.First),
	  Last=(Item==Block.Last);
  uint PosX;
  uint Width=B.X-A.X-2;
  string Str=new_str_cpy (Item->Value);
  // 1) Ma se vubec neco vykreslovat??
  if ((Last&&Block.BEnd>=FirstCh.X)||
     (First&&Block.BBeg<FirstCh.X+Width)){
    // 2) Urceni posice na obrazovce
    if (First)
      PosX=max (0,(int)Block.BBeg-FirstCh.X);
    else
      PosX=0;
    // 3) Urceni zacatku stringu
    if (Last) {
      Str[Block.BEnd+1]='\0';
      if (!First) Str=&Str[FirstCh.X];
    }
    if (First) {
      if (FirstCh.X>Block.BBeg){
	if (strlen (Str)>FirstCh.X)
	  Str=&Str[FirstCh.X];
	else Str=NULL;
      }
      else
	Str=&Str[Block.BBeg];
    }
    if (Str!=NULL)
    {
    // 4) Urceni konce stringu (delky)
    if (strlen(Str)>Width-PosX)
      Str[Width-PosX]='\0';
    if (strlen (Str)>0 && Str[strlen(Str)-1]=='\n')
      Str[strlen(Str)-1]='\0';
    // 5) Vlastni vykresleni
    goto_xy (A.X+1+PosX,A.Y+1+Y);
    block_attr();
    cprintf ("%s",Str);
    normal_attr ();
    }
  }
  free (Str);
} /* TEdit::write_block_line */

//--------------------------- TEdit::del_line  -----------------------------------------
void TEdit::del_line (uint Y) {
  // smaze radek Y na obrazovce
  print_xyl (A.X+1,A.Y+1+Y,B.X-A.X-2,"                                                                              ");
  //"");
} /* TEdit::del_line */

//--------------------------- TEdit::akt_pos  -----------------------------------------
TPoint TEdit::akt_pos () {
  TPoint Pom;
  Pom.X=CurPos.X+FirstCh.X;
  Pom.Y=CurPos.Y+FirstCh.Y;
  return (Pom);
} /* TEdit::akt_pos */

//--------------------------- TEdit::write_lines  -----------------------------------------
void TEdit::write_lines () {
  // Vykresli vsechny aktualne videne radky
  TAttrCur AC=save_ac();
  normal_attr();

  //TCursor Cur=save_cursor();
  uint Max=B.Y-A.Y-2;
  TStringItem *Pom=Lines->p_item (FirstCh.Y);
  uint i=0;

  if (Pom!=NULL){
    for(i=0; (i<Max && Pom!=NULL); i++){
      if (BlockSelected && FirstCh.Y+i>BBeg && FirstCh.Y+i<BBeg+Block.Count-1){
	p_write_line (Pom,i,TRUE);
      }
      else{
	p_write_line (Pom,i);
	if (BlockSelected && (FirstCh.Y+i==BBeg || FirstCh.Y+i==BBeg+Block.Count-1))
	  write_block_line (Pom,i);
      }
      Pom=Pom->Next;
    }
  }
  if (i<Max)
    for(;i<Max; i++)
      del_line (i);

  restore_ac (AC);
  goto_xy (A.X+1+CurPos.X, A.Y+1+CurPos.Y);
} /* TEdit::write_lines */
//--------------------------- TEdit::normal_attr  -----------------------------------------
void TEdit::normal_attr () {
  textattr (attrib (BackC,LetterC));
} /* TEdit::normal_attr */
//--------------------------- TEdit::block_attr  -----------------------------------------
void TEdit::block_attr () {
  textattr (attrib (BlockBackC,BlockLetterC));
} /* TEdit::block_attr */




//--------------------------- Procedury a fce na pohyb cursoru ---------------



//--------------------------- TEdit::cur_lr  -----------------------------------------
void TEdit::cur_lr (int Step) {
// posune cursor o Step kroku doprava (Step>0), nebo doleva (Step<0)
  uint PosX=FirstCh.X+CurPos.X;
  uint Width=B.X-A.X-2;

  if (Step>0 && PosX+Step<MAX_CHARS_PER_LINE){
    if (CurPos.X+Step<Width) CurPos.X+=Step;
    else {
      FirstCh.X+=(Step-(Width-CurPos.X-1));
      CurPos.X=Width-1;
      write_lines();
    }
  }
  else if (Step<0 && PosX>0){
    if (CurPos.X+Step>=0)
      CurPos.X+=Step;
    else{
      FirstCh.X-=(-CurPos.X-Step);
      if (FirstCh.X<0) FirstCh.X=0;
      CurPos.X=0;
      write_lines();
    }
  }
  write_cur_pos();
} /* TEdit::cur_lr */

//--------------------------- TEdit::cur_ud  -----------------------------------------
void TEdit::cur_ud (int Step) {
  uint PosY=FirstCh.Y+CurPos.Y;
  uint Height=B.Y-A.Y-2;
  if (Step>0 && PosY<Lines->Count){
    // cursor down
    if (PosY+Step>=Lines->Count)
      Step=Lines->Count-PosY-1;

    if (CurPos.Y+Step<Height)
      CurPos.Y+=Step;
    else{
      FirstCh.Y+=(Step-(Height-CurPos.Y-1));
      CurPos.Y=Height-1;
      write_lines();
    }
  }
  else if (Step<0){
    Step=-Step;
    if (PosY<Step) Step=CurPos.Y+FirstCh.Y;
    if (CurPos.Y>=Step)
      CurPos.Y-=Step;
    else {
      FirstCh.Y-=(Step-CurPos.Y);
      if (FirstCh.Y<0) FirstCh.Y=0;
      CurPos.Y=0;
      write_lines();
    }
  }
  write_cur_pos();
} /* TEdit::cur_ud */

//--------------------------- TEdit::handle_event  -----------------------------------------
void TEdit::handle_event (int Key) {
  TPoint Pos=akt_pos();
  string Str=NULL;
  uint Height=B.Y-A.Y-2;
  switch (Key) {

    // I) POSUN KURSORU
    case K_LEFT: cur_lr (-1); break;
    case K_RIGHT: cur_lr (1); break;
    case K_UP: cur_ud (-1); break;
    case K_DOWN: cur_ud (1); break;
    case K_ALT_LEFT: cur_lr (-4); break;
    case K_ALT_RIGHT: cur_lr (4); break;
    case K_ALT_UP: cur_ud (-4); break;
    case K_ALT_DOWN: cur_ud (4); break;

    case K_CTRL_RIGHT:
      Str=Lines->item(Pos.Y);
      cur_lr (next_word(Str,Pos.X));
    break;
    case K_CTRL_LEFT:
      Str=Lines->item(Pos.Y);
      cur_lr (-prev_word(Str,Pos.X));
    break;
    case K_CTRL_UP:
      if (FirstCh.Y>0 && CurPos.Y<Height-1) {
	FirstCh.Y-=1;
	CurPos.Y+=1;
	write_lines();
      }
    break;
    case K_CTRL_DOWN:
      if (CurPos.Y>0 && Pos.Y<Lines->Count){
	FirstCh.Y+=1;
	CurPos.Y-=1;
	write_lines();
      }
    break;

    //PgUpDown,Home,End,Ctrl+...
    case K_HOME: cur_lr (-Pos.X); break;
    case K_END: cur_lr (strlen (Str=Lines->item(Pos.Y))-Pos.X-1); break;
    case K_CTRL_HOME: cur_ud (-CurPos.Y);break;
    case K_CTRL_END: cur_ud (Height-CurPos.Y-1);break;
    case K_PGDN:
      FirstCh.Y+=Height-1;//cur_ud (Height-1); break;
      if (FirstCh.Y>=Lines->Count) {
	FirstCh.Y=Lines->Count-1;
	if (FirstCh.Y+CurPos.Y>=Lines->Count)
	  CurPos.Y=Lines->Count-FirstCh.Y;
      }
      write_lines();
      write_cur_pos ();
    break;

    case K_PGUP: //cur_ud (-(Height-1));
      if (FirstCh.Y<Height-1) {
	FirstCh.Y=0;
	CurPos.Y=0;
      }
      else FirstCh.Y-=Height-1;
      write_lines ();
      write_cur_pos ();
    break;
    case K_CTRL_PGUP:cur_ud(-Pos.Y);break;
    case K_CTRL_PGDN:cur_ud (Lines->Count-Pos.Y);break;

    // II) Operace s bloky
    case K_F4: select_block (); break;
    case K_ESC:
      if (BlockSelected){
	deselect_block ();
	write_lines ();
      }
    break;

    case K_CTRL_A: b_upcase(); break;
    case K_CTRL_Z: b_downcase(); break;
    case K_CTRL_I: b_indent(); break;
    case K_CTRL_U: b_unindent(); break;
    case K_CTRL_C: clip_copy(); break;
    case K_CTRL_V: clip_paste(); break;
    case K_CTRL_X: clip_cut(); break;
    case K_CTRL_D: b_del(); break;


    // III) Ostatni operace (Load,save,new,...)
    case K_F2: save(); break;
    case K_ALT_F2: save_as (); break;
    case K_CTRL_F: find (); break;
    case K_CTRL_L: cur_lr (1); find_again (); break;
    case K_CTRL_R: replace(); break;

    // IV) Operace editacni
    case K_CTRL_Y: remove_line(); break;
    case K_INS: set_insert_mode (!InsertMode); break;
    case K_ENTER: handle_enter_key (); break;
    case K_DEL: handle_del_key (); break;
    case K_BKSP: handle_bksp_key (); break;
    case K_CTRL_S: change_syntax(); break;
    case K_CTRL_Q: ST->clear(); write_lines(); break;
    case K_CTRL_T: del_word (); break;
    default:
      if ((Key=='A'|| Key=='a') && OldKey==K_CTRL_Q)
	replace();
      else
	insert_ch (Key);
    break;
  }
  OldKey=Key;
} /* TEdit::handle_event */




//--------------------------- Procedury a fce pro praci s   B L O K Y  -----------------------------------------



//--------------------------- TEdit::deselect_block  -----------------------------------------
void TEdit::deselect_block () {
  BlockSelected=FALSE;
  SelectingBlock=FALSE;
  BBeg=0;
  BBegPos=0;
  Lines->b_reset(Block);
} /* TEdit::deselect_block */

//--------------------------- TEdit::select_block  -----------------------------------------
void TEdit::select_block () {
  TPoint Pos=akt_pos();
  if (SelectingBlock){
    // nastavi druhy konec bloku, cimz jest cely blok urcen
    BlockSelected=TRUE;
    SelectingBlock=FALSE;
    Block=Lines->b_set (BBeg,Pos.Y,BBegPos,Pos.X);
    write_lines();  // vykresleni bloku
  }
  else {
    if (BlockSelected) {
      // Nejdrive zrusi stary blok
      deselect_block();
      write_lines();  // prekresleni bloku
    }
    // nastavi teprve zacatek bloku
    SelectingBlock=TRUE;
    BBeg=Pos.Y;     // radek
    BBegPos=Pos.X;  // posice v radku
  }
} /* TEdit::select_block */
//--------------------------- TEdit::b_del  -----------------------------------------
void TEdit::b_del () {
  TPoint Pos=akt_pos();
  if (BlockSelected){
    // presune se kursor na spravne misto, ktere je jeste ,,v textu''

    cur_lr(Block.BBeg-Pos.X-1);
    cur_ud(BBeg-Pos.Y);
    Lines->b_delete(Block);
    if (CurPos.Y+FirstCh.Y>=Lines->Count)
      cur_ud (-1);
    deselect_block();
    check_lines();
    write_lines();
    set_modify (TRUE);
  }
} /* TEdit::b_del */
//--------------------------- TEdit::b_indent  -----------------------------------------
void TEdit::b_indent () {
  if (BlockSelected){
    Lines->b_indent (Block);
    write_lines ();
    set_modify (TRUE);
  }
} /* TEdit::b_indent */
//--------------------------- TEdit::b_unindent  -----------------------------------------
void TEdit::b_unindent () {
  if (BlockSelected){
    Lines->b_unindent (Block);
    write_lines ();
    set_modify (TRUE);
  }
} /* TEdit::b_unindent */

//--------------------------- TEdit::b_upcase  -----------------------------------------
void TEdit::b_upcase () {
  if (BlockSelected){
    Lines->b_case (Block, TRUE);
    write_lines ();
    set_modify (TRUE);
  }
} /* TEdit::b_upcase */
//--------------------------- TEdit::b_downcase  -----------------------------------------
void TEdit::b_downcase () {
  if (BlockSelected){
    Lines->b_case (Block, FALSE);
    write_lines ();
    set_modify (TRUE);
  }
} /* TEdit::b_downcase */
//--------------------------- TEdit::clip_copy  -----------------------------------------
void TEdit::clip_copy () {
  if (BlockSelected){
    Lines->b_copy (Block,ClipBoard);
  }
} /* TEdit::clip_copy */
//--------------------------- TEdit::clip_paste  -----------------------------------------
void TEdit::clip_paste () {
  TPoint Pos=akt_pos();
  if (!ClipBoard->empty()){
    deselect_block();
    Lines->b_paste (ClipBoard,Lines->p_item(Pos.Y),Pos.X);
    write_lines();
    set_modify (TRUE);
  }
} /* TEdit::clip_paste */
//--------------------------- TEdit::clip_cut  -----------------------------------------
void TEdit::clip_cut () {
  if (BlockSelected){
    clip_copy();
    b_del();
  }
} /* TEdit::clip_cut */


//--------------------------- Procedury a fce na EDITACI textu  -----------------------------------------


//--------------------------- TEdit::remove_line  -----------------------------------------
void TEdit::remove_line () {
  uint PosY=FirstCh.Y+CurPos.Y;
  if (BlockSelected)
    deselect_block();
  Lines->remove(PosY);
  check_lines ();
  if (PosY>=Lines->Count)
    cur_ud (-1);
  write_lines();
  set_modify (TRUE);
} /* TEdit::remove_line */
//--------------------------- TEdit::set_insert_mode  -----------------------------------------
void TEdit::set_insert_mode (boolean Mod) {
  if (Mod!=InsertMode){
    InsertMode=Mod;
    if (InsertMode)
      set_cursor (CUR_INSERT);
    else
      set_cursor (CUR_OVER);
  }
} /* TEdit::set_insert_mode */
//--------------------------- TEdit::insert_ch  -----------------------------------------
void TEdit::insert_ch (int Key) {
  if (Key>31 && Key<256){
    if (BlockSelected){
      deselect_block();
      write_lines();
    }
    TPoint Pos=akt_pos();
    TStringItem *Item=Lines->p_item (Pos.Y);

    if (!InsertMode && Pos.X<strlen (Item->Value)-1)
      // prepisovaci mod
      Lines->del_ch (Item, Pos.X);

    // vkladaci mod
    Lines->insert_ch(Item,Pos.X,Key);
    cur_lr (1);
    TAttrCur AC=save_ac ();
    normal_attr ();
    p_write_line (Item,CurPos.Y);
    restore_ac (AC);
    set_modify (TRUE);
  }
} /* TEdit::insert_ch */
//--------------------------- TEdit::handle_enter_key  -----------------------------------------
void TEdit::handle_enter_key () {
  if (BlockSelected){
    deselect_block();
    write_lines();
  }

  TPoint Pos=akt_pos();
  TStringItem *Item=Lines->p_item (Pos.Y);
  if (Pos.X<strlen(Item->Value))
    Lines->p_split (Item, Pos.X);
  else
    Lines->p_insert (new_str_cpy("\n"),Item->Next);
  write_lines();
  cur_ud (1);
  cur_lr (-Pos.X);
  set_modify (TRUE);
} /* TEdit::handle_enter_key */
//--------------------------- TEdit::handle_del_key  -----------------------------------------
void TEdit::handle_del_key () {
  if (BlockSelected){
    deselect_block();
    write_lines();
  }
  TPoint Pos=akt_pos();
  TStringItem *Item=Lines->p_item (Pos.Y);
  if (Pos.X<strlen (Item->Value)-1){
    Lines->del_ch (Item,Pos.X);
    TAttrCur AC=save_ac ();
    normal_attr();
    p_write_line (Item,CurPos.Y);
    restore_ac (AC);
  }
  else if (Item!=Lines->Tail){
    string Str=Item->Value;
    uint N=strlen (Str);
    uint Tab=Pos.X-(N-1);
    string Spaces=n_char (Tab,' ');
    if (strlen (Item->Next->Value)>1)
      Item->Value=replace_str (Spaces,Str,N-1,0);
    free (Spaces);
    Lines->p_join (Item);
    write_lines ();
  }
  set_modify (TRUE);
} /* TEdit::handle_del_key */
//--------------------------- TEdit::handle_bksp_key  -----------------------------------------
void TEdit::handle_bksp_key () {
  if (BlockSelected){
    deselect_block();
    write_lines();
  }

  TPoint Pos=akt_pos();
  TStringItem *Item=Lines->p_item (Pos.Y);
  if (Pos.X>0){
    if (Pos.X>=strlen (Item->Value))
      cur_lr (-1);
    else {
      Lines->del_ch (Item,Pos.X-1);
      TAttrCur AC=save_ac ();
      normal_attr();
      p_write_line (Item,CurPos.Y);
      restore_ac (AC);
      cur_lr (-1);
    }
  }
  else if (Item!=Lines->Head && Pos.X==0){
    uint Tab=strlen (Item->Prev->Value);
    Lines->p_join (Item->Prev);
    write_lines ();
    cur_ud (-1);
    cur_lr (Tab-1);
  }
  set_modify (TRUE);
} /* TEdit::handle_bksp_key */

//--------------------------- TEdit::check_lines  -----------------------------------------
void TEdit::check_lines () {
  if (Lines->empty())
    Lines->add(new_str_cpy ("\n"));
} /* TEdit::check_lines */

//--------------------------- TEdit::in_block  -----------------------------------------
boolean TEdit::in_block (TPoint Pos) {
  uint BEndPos=Block.BEnd;
  long BEnd=BBeg+Block.Count;

  if (BlockSelected &&
     ((Pos.Y>BBeg && Pos.Y<BEnd)||       // je mezi
     (Pos.Y==BBeg==BEnd && Pos.X>=BBegPos && Pos.X <=BEndPos) ||//
     (Pos.Y==BBeg && Pos.X>=BBegPos) ||//
     (Pos.Y==BEnd && Pos.X <=BEndPos)))
    return (TRUE);
  else
    return (FALSE);
} /* TEdit::in_block */
//--------------------------- TEdit::find_again  -----------------------------------------
void TEdit::find_again () {
  string Str;
  boolean CS,WW,Global;
  get_find_dlg_opt (Str,CS,WW,Global);
  find_string (Str,CS,WW,Global);
} /* TEdit::find_again */
//--------------------------- TEdit::find_string  -----------------------------------------
int TEdit::find_string (string Str,boolean CS, boolean WW, boolean Global, boolean Show) {
  if (strcmp (Str,"") && (Global||(!Global&&BlockSelected))){
    TPoint Pos=akt_pos();

    if (!Global && !in_block (Pos)){
      Pos.X=BBegPos;
      Pos.Y=BBeg;
    }
    TStringItem *Pom=Lines->p_item (Pos.Y);
    string Line=Pom->Value;
    uint Start=min (Pos.X,strlen (Line)-1);
    long Step=Pos.Y;
    int Res;

    while ((Step<Lines->Count) &&
	   (Res=find_str(Str,Line,Start,CS,WW))==-1){
      Pom=Pom->Next;
      Line=Pom->Value;
      Step++;
      Start=0;
    }
    if (Res!=-1 && (Global||(!Global && (Step<BBeg+Block.Count||(Step==BBeg+Block.Count && Res<=Block.BEnd))))){
      // Nalezen
      Pos=akt_pos();
      cur_ud (Step-Pos.Y);
      cur_lr (Res-Pos.X);
      return (0);
    }
  }
  // Nenalezen
  if (Show)
    message_dlg ("        Hledany vzorek nebyl nalezen",1,1);
  return (-1);
} /* TEdit::find_string */
//--------------------------- TEdit::find  -----------------------------------------
void TEdit::find () {
  string Str;
  boolean CS,WW,Global;
  if (open_find_dlg (Str,CS, WW, Global))
    find_string(Str,CS, WW, Global);
} /* TEdit::find */
//--------------------------- TEdit::replace  -----------------------------------------
void TEdit::replace () {
  string Str1, Str2;
  boolean CS,WW,Global;
  uint R;
  boolean Show=TRUE;
  if ((R=open_replace_dlg (Str1,Str2,CS,WW,Global))!=0 &&
      strcmp (Str1,"")){
    TPoint Pos;
    TStringItem *Item;
    string Where;

    while (find_string (Str1,CS,WW,Global,Show)!=-1 && R!=0){
      Pos=akt_pos();
      Item=Lines->p_item (Pos.Y);
      Where=Item->Value;
      Item->Value=replace_str (Str2,Where,Pos.X,strlen (Str1));
      set_modify (TRUE);
      if (R==1) R=0;
      else {
	cur_lr (strlen (Str2));
      Show=FALSE;
      }
    }
    deselect_block ();
    write_lines();
  }
} /* TEdit::replace */

//--------------------------- TEditor::help  -----------------------------------------
void TEditor::help () {
  message_dlg (
    "            POHYB KURSORU V TEXTU           "
    "Sipky (+CTR, +ALT), PgUp, PgDown, Home, End,"
    "CTR+PgUp, CTR+PgDown, CTR+Home, CTR+End     "
    "                                            "
    "              OPERACE S BLOKEM              "
    "F4 Oznaceni   CTR+C Kopirovani bloku do clip"
    "ESC Zruseni   CTR+V Vlozeni clip. do textu  "
    "CTR+D Smazani CTR+X Presunuti bloku do clip."
    "CTR+A (CTR+Z) UpCase(DownCase) bloku        "
    "CTR+U (CTR+I) (Un)Indentace bloku           "
    "                                            "
    "               OSTATNI FUNKCE               "
    "CTR+Y (T) Smaze radku (slovo) INS Mod klav. "
    "CTR+F3 Novy dokument  F3 Otevre dokument    "
    "F2 Ulozi dokument  ALT+F2 Prejmenuje a ulozi"
    "CTR+F2 Ulozi vsechny dokumenty              "
    "F1 Tato napoveda   ALT+F1 Dalsi informace   "
    "CTR+F (L)/CTR+Q+A Hledani  CTR+R  Nahrazeni "
    "CTR+S Nastaveni syntaxe CTR+Q Vypnuti synt. "
    "ALT+X Ukonci prg. Dalsi info viz dokumentce"
    ,1,1);

} /* TEdit::help */
//--------------------------- TEditor::info  -----------------------------------------
void TEditor::info () {
  message_dlg (
    "     Programatorsky EDITOR  (verse 2.1)     "
    "                                            "
    "Tato aplikace vznikla jako zapoctovy program"
    "do predmetu PROGRAMOVANI V C/C++  a PRAKTIKA"
    "Z PROGRAMOVANI (4.semestr MFF UK oboru Inf)."
    "                                            "
    "Autor programu:                             "
    "Petr Vaclavek, Svobodova 145, Vidnava 79055 "
    "                                            "
    "                    ve Vidnave 14. zari 1997"
    ,1,1);


} /* TEditor::info */

//--------------------------- Tedit::change_syntax  -----------------------------------------
void TEdit::change_syntax () {
  string NewSFile;
  int Res;
  int Synt;
  string FileN;

  Res=open_syntax_dlg (Synt,NewSFile);
  if (Res){
    if (Res==5){
      // Syntaxi vypnout
      ST->clear();
      write_lines();
    }
    else{
      // Syntaxi zmenit
      switch (Synt) {
	case 0: FileN="s_table.ini"; break;
	case 1: FileN="c_table.ini"; break;
	case 2: FileN="h_table.ini"; break;
	case 3: FileN="pr_table.ini"; break;
	case 4: FileN=NewSFile; break;
      } // end of switch
      load_syntax (FileN);
      write_lines();
    }
  }
} /* Tedit::change_syntax */
//--------------------------- TEdit::load_syntax  -----------------------------------------
void TEdit::load_syntax (string FileN) {
  int Res=ST->load (FileN);
  string Message;
  switch (Res) {
    case -1: Message="      Nepodarilo se otevrit INI-soubor"; break;
    case -2: Message="    Malo pameti pro inicialisaci syntaxe"; break;
    case -3: Message="        Spatna struktura INI-souboru"; break;
  }
  if (Res)
    message_dlg (Message,1,1);
  } /* TEdit::load_syntax */
//--------------------------- TEdit::del_word  -----------------------------------------
void TEdit::del_word () {
  char W[MAX_CHARS_PER_LINE];
  string Word=W;
  TPoint Pos=akt_pos ();
  string Where=Lines->item (Pos.Y);
  uint Length=1;

  if (read_word (Where,Pos.X,Word))
    Length=strlen (Word);

  for(int i=0; i<Length; i++)
    handle_del_key();

} /* TEdit::del_word */




//--------------------------- TEditor::TEditor  -----------------------------------------
TEditor::TEditor (string FileName) {
  Head=new_edit (FileName);
  AktItem=Head;
  Count=1;
  EditorResult=-1;
} /* TEditor::TEditor */
//--------------------------- TEditor::new_edit  -----------------------------------------
TEditItem *TEditor::new_edit (string FileName) {
  TEditItem *EditItem=(TEditItem *)malloc (sizeof (TEditItem));
  TEdit *_Edit=new TEdit (point (0,0),point (MAX_X,MAX_Y),FileName);
  EditItem->Edit=_Edit;
  EditItem->Next=NULL;
  EditItem->Edit->draw();
  return (EditItem);
} /* TEditor::new_edit */

//--------------------------- TEditor::next_item  -----------------------------------------
TEditItem *TEditor::next_item (TEditItem *Item) {
  if (!Count) return (NULL);
  if (Item->Next!=NULL)
    return (Item->Next);
  else
    return (Head);
} /* TEditor::next_item */
//--------------------------- TEditor::prev_item  -----------------------------------------
TEditItem *TEditor::prev_item (TEditItem *Item) {
  // pomoci next item
  if (!Count) return (NULL);
  for(uint i=0; i<Count-1; i++)
    Item=next_item (Item);
  return (Item);
} /* TEditor::prev_item */
//--------------------------- TEditor::insert_item  -----------------------------------------
void TEditor::insert_item (TEditItem *Item) {
  // vlozi za aktualni posici urcenou promennou AktItem polozku Item
  // a prepne se na nej
  if (Item!=NULL){
    if (!Count){
      Head=Item;
      AktItem=Item;
    }
    else{
      Item->Next=AktItem->Next;
      AktItem->Next=Item;
      AktItem=Item;
    }
    Count++;
    AktItem->Edit->draw();
  }
} /* TEditor::insert_item */

//--------------------------- TEditor::remove_item  -----------------------------------------
void TEditor::remove_item (){// (TEditItem *Item) {
  TEditItem *Item=AktItem;
  if (Item!=NULL){
    if (Item==Head){
      Head=Item->Next;
      AktItem=Head;
    }
    else{
      TEditItem *Prev=prev_item (Item);
      Prev->Next=Item->Next;
      AktItem=Prev;
    }

    delete Item->Edit;
    free (Item);
    Count--;
    if (AktItem!=NULL)
      AktItem->Edit->draw ();
    else
      draw_background();
  }
} /* TEditor::remove_item */

//--------------------------- TEditor::handle_event  -----------------------------------------
void TEditor::handle_event (int Key) {
  switch (Key) {
    case K_ALT_F3:
      if (AktItem!=NULL)
	if (AktItem->Edit->close()!=-1)
	  remove_item ();
    break;
    case K_F3:
      insert_item (new_edit ());
      AktItem->Edit->load();
    break;
    case K_CTRL_F3:
      insert_item (new_edit ());
    break;

    case K_ALT_F6:
    case K_F6:
      if (AktItem!=NULL){
	TEditItem *OldEdit=AktItem;
	if (Key==K_F6)
	  AktItem=next_item (AktItem);
	else
	  AktItem=prev_item (AktItem);
	if (OldEdit!=AktItem)
	  AktItem->Edit->draw();
      }
    break;

    case K_CTRL_F2:
      if (Count>0){
	TEditItem *OldAkt=AktItem;
	while ((AktItem->Edit->save()!=0) &&
	(AktItem=next_item (AktItem))!=OldAkt)
	  AktItem->Edit->draw();
	AktItem->Edit->draw();
      }
    break;

    case K_ALT_X:
      while (Count>0){
	if (AktItem->Edit->close()!=-1)
	  remove_item ();
	else
	  break;
      }
      if (Count==0)
	EditorResult=0;
    break;
    case K_F1: help(); break;
    case K_ALT_F1: info(); break;

    default:
      if (AktItem!=NULL) AktItem->Edit->handle_event (Key);
    break;
  }
} /* TEditor::handle_event */
//--------------------------- TEditor::run  -----------------------------------------
void TEditor::run () {
  int (Key);
  while (EditorResult==-1){
    Key=read_key();
    handle_event (Key);
  }
  textattr (7);
  clrscr();
  set_cursor (CUR_INSERT);
  printf ("Diky za pouziti tohoto editoru...\n");
} /* TEditor::run */
//--------------------------- TEditor::draw_background  -----------------------------------------
void TEditor::draw_background () {
  textattr (7);
  clrscr();
  set_cursor (CUR_OFF);
  for(uint i=0; i<MAX_X; i++)
    for(uint j=0; j<MAX_Y; j++)
      put_char (i,j,'');//random ('Z'-'A')+'A');
  print_xyl (0,0,MAX_X,"          P R O G R A M A T O R S K Y     E D I T O R  (verse 2.1)");
  print_xyl (0,MAX_Y-1,MAX_X-1,"   F1 Napoveda (ovladani)   ALT+F1 Dalsi informace");
  put_char_attr (MAX_X-1,MAX_Y-1,' ',0);

} /* TEditor::draw_background */

//===========================  HLAVNI PROGRAM  ===============================


void main (int Argc, string Argv[]) {
  clrscr();
  // priprava dialogu
  init_choosefile_dlg ();
  init_find_dlg ();
  init_replace_dlg ();
  init_syntax_dlg ();
  ClipBoard=new TStrings();

  string InputFile=NULL;
  if (Argc>1)
    InputFile=Argv[1];

  TEditor *Editor=new TEditor(InputFile);
  Editor->run();
  delete Editor;
  delete ClipBoard;

  //uvolneni dialogu
  free_choosefile_dlg ();
  free_find_dlg ();
  free_replace_dlg ();
  free_syntax_dlg ();
}

