#ifndef _EDITOR_
  #define _EDITOR_

  #ifndef _TYPES_
    #include "types.h"
  #endif

  #ifndef _GRAPHIC_
    #include "graphic.h"
  #endif

  #ifndef _STRING_
    #include "string.h"
  #endif

  #ifndef _KEYBOARD_
    #include "keyboard.h"
  #endif

  #ifndef _WINDOWS_
    #include "windows.h"
  #endif

  #ifndef _SYNTAX_
    #include "syntax.h"
  #endif

#define MAX_CHARS_PER_LINE 300
#include<dir.h>

typedef struct SStringItem TStringItem;
struct SStringItem {
  string Value;
  struct SStringItem *Next, *Prev;
};

typedef struct {
  TStringItem* First;   // Ukazatel na prvni polozku bloku
  TStringItem* Last;    // Ukazatel na posledni polozku bloku
  uint BBeg;            // Index prvniho pismene bloku
  uint BEnd;            // Index posledniho pismene bloku
  uint Count;           // Pocet polozek bloku
} TBlock;


//--------------------------- TStrings -----------------------------------------
class TStrings {
  public:
    TStringItem *Head,*Tail;
    long Count;

  // METODY
    TStrings();
    ~TStrings();

    TStringItem *new_item (string Value);
    void add (string Value);

    TStringItem *p_insert (string Value, TStringItem *NextItem);
    TStringItem *insert (string Value, long ItemIndex);

    void p_exchange (TStringItem *Item1, TStringItem *Item2);
    void exchange (long Itemindex1, long ItemIndex2);

    TStringItem *p_cut (TStringItem *Item);
    TStringItem *cut (long ItemIndex);
    void remove (long ItemImdex);
    void p_remove (TStringItem *Item);

    TStringItem *p_item (long ItemIndex);
    string item (long ItemIndex);
    TStringItem *next_item (TStringItem *Item);

    TStringItem *p_split (TStringItem *Item, uint Pos);
    TStringItem *split (long ItemIndex, uint Pos);
    void p_join (TStringItem *FirstItem);
    void join (long FirstItemIndex);

    void print ();
    int load (string FileName);
    int save (string FileName);
    void clear ();
    boolean empty(){return ((Head==NULL&&Tail==NULL)?TRUE:FALSE);};


   // blokove operace
    TBlock b_set (long &First, long Last, uint &BBeg, uint BEnd);
    void TStrings::b_reset (TBlock &Block);

    void b_prepare (TBlock Block, string &FBLine, char &LBChar);
    void b_restore (TBlock Block, string FBLine, char LBChar);
    void b_print (TBlock Block);
    void b_case (TBlock Block, boolean UpCase);
    void b_indent (TBlock &Block);
    void b_unindent (TBlock &Block);
    void b_delete (TBlock &Block);
    void b_copy (TBlock Block, TStrings *OutList);
    void b_paste (TStrings *From, TStringItem *Where, uint Pos);

    void insert_ch (TStringItem *Where, uint Pos, char Z);
    void del_ch (TStringItem *Where, uint Pos);
};


//--------------------------- TEdit -----------------------------------------------

class TEdit {
  protected:
    uint BackC,   // barva pozadi
	 FrameC,        // ramecku
	 LetterC,       // pisma
	 BlockBackC,    // pozadi bloku
	 BlockLetterC;   // pisma bloku
    TPoint A,B;             // poloha a rozmery
    TPoint CurPos;          // posice kursoru vzhledem k obrazovce (0-25*0-80)
    TPoint FirstCh;         // posice aktualniho vyrezu textu
    boolean InsertMode;     // mod klavesnice (prepisovani/vkladani)
    boolean BlockSelected;  // zda je oznacen nejaky blok
    boolean SelectingBlock; // zda se prave oznacuje nejaky blok
    long BBeg;              // zacatek bloku pri oznacovani (cislo radku)
    uint BBegPos;            // zacatek bl. pri oznacovani (posice na radku)
    TBlock Block;           // prave oznaceny blok
    boolean Modify;         // zda doslo k nejake modifikaci textu
    TStrings *Lines;        // vlastni radky textu
    char FileName[MAXPATH]; // jmeano aktualne editovaneho souboru
//    TStrings *ClipBoard;    // clipboard pro praci s blokem textu
    int EditResult;         // vysledek editace (ukoncovaci promenna)
    THScrollBar *HScrollBar; // Horizontalni ukazatel polohy cur. v textu
    TVScrollBar *VScrollBar; // Vertikalni .....
    TSyntaxTable *ST;        // Tabulka syntaxe
    int OldKey;              // Minula klavesa

  public:
    TEdit (TPoint _A, TPoint _B, string _FileName=NULL);
    void run ();
    int close ();
    void draw ();
    void handle_event (int Key);

    void load (string _FileName=NULL);
    int save ();
    int save_as ();
    int save_changes ();
    void clear ();
    void prepare_lines ();
//    void help();
//    void info();

    int find_string (string Str,boolean CS,boolean WW,boolean Global, boolean Show=TRUE);
    void find ();
    void find_again ();
    void replace ();

    void set_modify (boolean _Modify);
    void set_insert_mode (boolean Mod);
    void set_defaults ();
    TPoint akt_pos ();

    void remove_line();
    void insert_ch (int Key);
    void handle_enter_key ();
    void handle_del_key ();
    void handle_bksp_key ();
    void check_lines ();

    void cur_lr (int Step);
    void cur_ud (int Step);

    void normal_attr ();
    void block_attr ();
    void del_line (uint Y);
    void write_lines ();
    void p_write_line (TStringItem *Item, uint Y, boolean Bl=FALSE);
    void write_block_line (TStringItem *Item, uint Y);
    void write_file_name ();
    void write_cur_pos ();
    void write_modify ();

    void select_block ();
    void deselect_block ();
    void p_write_block_line (TStringItem *Line, long LineNum, uint Y);
    void b_indent ();
    void b_unindent ();
    void b_upcase ();
    void b_downcase ();
    void b_del ();
    boolean in_block (TPoint Pos);

    void clip_copy();
    void clip_paste();
    void clip_cut();
    void change_syntax();
    void load_syntax (string FileN);
    void del_word ();
};

typedef struct SEditItem TEditItem;
struct SEditItem {
  TEdit * Edit;
  struct SEditItem *Next;
};


class TEditor {
  public:
    TEditItem *Head;   // lin. seznam editacnich oken
    uint Count;        // pocet editacnich oken
    TEditItem *AktItem;// aktualni editacni okno
    int EditorResult;

    TEditor (string FileName=NULL);
    ~TEditor(){};
    TEditItem *prev_item(TEditItem *Item);
    TEditItem *next_item(TEditItem *Item);
    TEditItem *new_edit (string FileName=NULL); // vytvori novy editor

    void save_all ();
    void close_all ();
    void insert_item (TEditItem *Item);
    void remove_item ();//TEditItem *Item);
    void handle_event (int Key);
    void run ();
    void draw_background ();
    void help();
    void info();

};

#endif

